/*
 * Copyright (c) 2010 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 *
 * http://hardware.ispras.ru
 *
 * This code implements working with standard linux context.
 */

#include "sync.h"

#include <assert.h>
#include <stdlib.h>

#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <ucontext.h>
#elif defined(WIN32)
  #include <win32_ucontext/ucontext.h>
#else
  #error "Unknown platform"
#endif

#ifdef SIGSTKSZ
#undef SIGSTKSZ
#endif
#define SIGSTKSZ        (8192*32)

char stack[SIGSTKSZ];

ucontext_t scenario_context;
ucontext_t testbench_context;

static int(*__localscen)(int, char**);
static int __localargc;
static char **__localargv;

static int last_operation = 0;

static void scenario_wrapper(void)
{
    (*__localscen)(__localargc, __localargv);
}

int start_scenario(int (*scen)(int argc, char **argv), int argc, char **argv)
{
    assert(scenario_wrapper != NULL);

    __localscen = scen;
    __localargc = argc;
    __localargv - argv;

    getcontext(&scenario_context);
    scenario_context.uc_link = &testbench_context;
    scenario_context.uc_stack.ss_sp = stack;
    scenario_context.uc_stack.ss_size = SIGSTKSZ;
    makecontext(&scenario_context, scenario_wrapper, 0);
    swapcontext(&testbench_context, &scenario_context);

    return 0;
}

int end_scenario(void)
{
    return 0;
}

int wait_for_apply(void)
{
    return last_operation;
}

// scenario
void wait_for_check(void)
{
    swapcontext(&scenario_context, &testbench_context);
}

// testbench
void apply_check(void)
{
    swapcontext(&testbench_context, &scenario_context);
}

void apply_finish(void)
{
    last_operation = ACTION_FINISH_OPERATION;
}

void apply_operation(void)
{
    last_operation = ACTION_START_OPERATION;
}

void end_call(void)
{
}
